#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <sys/select.h>
#include <sys/time.h>
#include <pthread.h>
#include <tftpproxy.h>

static struct sockaddr_in __s_addr;
static struct sockaddr_in __p_addr;
static struct timeval __to;

static int __verbose = 0;

static int __lose_prb = LOST_PRB; /*lose propability*/
static int __lose_burst_len = LOST_BURST_LEN;
static int __repeat_prb = REPEAT_PRB;
static int __repeat_burst_len = REPEAT_BURST_LEN;
static int __repeat_port_switch = 0;

static int
__select(int rfd, int wfd, struct timeval* to)
{
	int maxfdp1 = 0;
	fd_set rset, wset;
	int ret = 0;

	FD_ZERO(&rset);
	FD_ZERO(&wset);
	if ( rfd > 0 ){
		DBG_MSG("Adding socket %d to read set.\n", rfd);	
		FD_SET(rfd,&rset);
	}
	if ( wfd > 0 ){
		DBG_MSG("Adding socket %d to write set.\n", wfd);	  
		FD_SET(wfd,&wset);
	}  
	maxfdp1 = max(rfd,wfd) + 1;
	DBG_MSG("Waiting for select.\n");    
	ret = select(maxfdp1, &rset, &wset, NULL, to);
	if ( ret > 0 ){
		DBG_MSG("Success in select.\n");
	}
	else if (ret == 0){ /* time out */
		DBG_MSG("Timeout while waiting on select\n");
	}
	else if (ret < 0) {
		DBG_ERR("Error in select.\n");
	}
	return ret;
}

static int
__select_read(int fd, struct timeval* to)
{
	return __select(fd, 0, to);
}

static int
__select_write(int fd, struct timeval* to)
{
	return __select(0, fd, to);
}


static ssize_t
__writen(int fd, uint8_t* buff, size_t nbytes)
{
	ssize_t nwritten;
	size_t nleft;
	uint8_t* buff_ptr;
    
	buff_ptr = buff;
	nleft = nbytes;
	while (nleft > 0){
		if ( (nwritten = write(fd, buff_ptr, nleft )) <= 0 ){
			if ( errno == EINTR ){
				nwritten = 0;
			}
			else{
				DBG_ERR("Error while writing to socket.\n");
				return -1;
			}
		}
		nleft -= nwritten;
		buff_ptr += nwritten;
	
	}
	return nbytes;
}

static void
__log_req(uint8_t* buff, size_t buff_len)
{
	char fname[FNAME_LEN + 1];
	char mode[MODE_LEN + 1];
	size_t str_len;

	if ( buff_len > 0 && buff_len <= (FNAME_LEN + MODE_LEN)){
		memset(fname, 0, sizeof(fname));
		memset(mode, 0, sizeof(mode));
		strncpy(fname, buff, min(buff_len, FNAME_LEN - 1));
		str_len = strlen(fname);
		if ( str_len > 0 && str_len < sizeof(fname) ){
			printf("Requested file is %s. ",fname);
			buff += str_len + 1;
			if ( buff_len > (str_len + 1)){
				buff_len-=(str_len + 1);
				strncpy(mode, buff, min(buff_len, MODE_LEN - 1));
				str_len = strlen(mode);
				if ( str_len > 0 && str_len < sizeof(mode) ){
					printf("Request mode %s.\n",mode);
				}
				else{
					printf("Invalid request mode of len %d.\n", str_len);
				}
			}
			else{
				printf("No more space for mode in request.\n");
			}
		}
		else{
			printf("Invalid file name of len %d in request buffer.\n", str_len);
		}
	}
	else{
		DBG_ERR("Buffer of length % d is too small for valid request packet.\n",
				buff_len);
	}
}

static void
__log_data_ack(uint8_t* buff, size_t buff_len)
{
	uint16_t blockno;

	if ( buff_len >= BLOCK_LEN ){
		memcpy(&blockno, buff, BLOCK_LEN);
		printf("Block number is %d.\n", ntohs(blockno));
	}
	else{
		DBG_ERR("Buffer of length % d is too small for valid data/ack packet.\n",
				buff_len);
	}
}

static void
__log_err(uint8_t* buff, size_t buff_len)
{
	uint16_t err_code;
	char err_msg[ERRMSG_LEN + 1];
	size_t err_msg_len;

	if ( buff_len >= ERRCODE_LEN ){
		memcpy(&err_code, buff, ERRCODE_LEN);
		printf("Error code is %d.\n", ntohs(err_code));
		if ( buff_len > ERRCODE_LEN ){
			buff+=ERRCODE_LEN;
			buff_len-=ERRCODE_LEN;
			memset(err_msg, 0, sizeof(err_msg));
			strncpy(err_msg, buff, min(buff_len, ERRMSG_LEN));
			err_msg_len = strlen(err_msg);
			if ( err_msg_len > 0 && err_msg_len < sizeof(err_msg) ){
				printf("Error message is %s.\n", err_msg);
			}
			else{
				printf("Invalid error message of len %d.\n", err_msg_len);
			}
		}
		else{
			printf("No error message.\n");
		}
	}
	else{
		DBG_ERR("Buffer of length % d is too small for valid error packet.\n",
				buff_len);
	}
  
}

static void
__log_buffer(uint8_t* buff, size_t buff_len)
{
	uint16_t opcode = 0;

	if ( __verbose > 1 ){ /* only at higher verbose levels */
		if ( buff ){
			printf("Type of packet is: ");
			if ( buff_len > OPCODE_LEN ){
				memcpy(&opcode, buff, OPCODE_LEN);
				opcode = ntohs(opcode);
				buff+=OPCODE_LEN;
				buff_len -= OPCODE_LEN;
				switch(opcode){
				case OP_RRQ:
					printf("Read Request packet. ");
					__log_req(buff, buff_len);
					break;
				case OP_WRQ:
					printf("Write Request packet. ");
					__log_req(buff, buff_len);
					break;
				case OP_DATA:
					printf("Data packet. ");
					__log_data_ack(buff, buff_len);
					break;
				case OP_ACK:
					printf("Ack packet. ");
					__log_data_ack(buff, buff_len);
					break;
				case OP_ERROR:
					printf("Error packet.\n ");
					__log_err(buff, buff_len);
					break;
				default:
					printf("Unknown. Opcode: %d\n",opcode);
				}

			}
			else{
				printf("No valid tftp packet.\n");
			}
		}
		else{
			DBG_ERR("Missing buffer to log.\n");
		}
	}
}

static int
__get_error(int prb, uint max_burst_len, int* burst)
{
	int z;

	if ( *burst > 0 ){
		DBG_MSG("Known burst len %d greater than zero.\n", *burst);
		(*burst)--; 
		return 1;
	}
	else if ( prb > 0 ){
		DBG_MSG("Propability is more than 0.\n");
		z = rand();
		if ( z <= (RAND_MAX / 100.0f * prb)){ /* random hit */
			/*
			  avoid floats. divide range into parts and look, where
			  the random number falls in
			 */
			*burst = rand() / ( RAND_MAX / (max_burst_len + 1) + 1);
			DBG_MSG("Random hit, new burst is %d\n", *burst);
			return 1;
		}
	}
	return 0;
}

static int
__send_error_phrone(int sockfd, uint8_t* buff, size_t buff_len, int* lburst)
{
	int tmpfd;
	struct sockaddr_in sock_addr, peer_addr;
	socklen_t sock_addr_len, peer_addr_len;
	char sock_addrp[INET_ADDRSTRLEN], peer_addrp[INET_ADDRSTRLEN];
	int n = 0;
	int sendings = 0;
	int add_repeats = 0;
	int pos = 0, port = 0, i= 0;

	sock_addr_len = sizeof(sock_addr);
	peer_addr_len = sizeof(peer_addr);
	if ( getsockname(sockfd,(struct sockaddr*)&sock_addr,&sock_addr_len ) < 0){
		DBG_ERR("Unable to get local address of client socket.\n");
		return 0;
	}
	
	if ( getpeername(sockfd,(struct sockaddr*)&peer_addr,&peer_addr_len ) < 0){
		DBG_ERR("Error getting peer address.\n");
		return -1;
	}
	if ( inet_ntop(AF_INET,
				   &sock_addr.sin_addr,
				   sock_addrp,
				   INET_ADDRSTRLEN) == NULL){
		DBG_ERR("Unable to convert sock address to presentationable.\n");
		return -1;
	}	
	if ( inet_ntop(AF_INET,
				   &peer_addr.sin_addr,
				   peer_addrp,
				   INET_ADDRSTRLEN) == NULL){
		DBG_ERR("Unable to convert peer address to presentationable.\n");
		return -1;
	}
	if ( __get_error(__lose_prb, __lose_burst_len, lburst) ){
		/* drop packet */
		DBG_MSG("Dropped packet. Lose burst is %d \n", *lburst);
		if ( __verbose ){
			printf("Dropped a packet on it's way from %s:%d to %s:%d\n",
				   sock_addrp,
				   ntohs(sock_addr.sin_port),
				   peer_addrp,
				   ntohs(peer_addr.sin_port));
			__log_buffer(buff, buff_len);
		}
	}
	else{
		sendings = 1; /* the packet will be sent */
		if ( __get_error(__repeat_prb, __repeat_burst_len, &add_repeats) ){
			/* send one more time, and the number of additional repeats */
			sendings+=(1 + add_repeats); 
		}
		DBG_MSG("Sending packet %d times \n", sendings);
		/* different behaviour, if port switching is allowed */
		if ( ! __repeat_port_switch ){ /* no port switching: simple write */
			for( i = 0; i < sendings; i++){
				n = __writen(sockfd, buff, buff_len);
				if ( n < buff_len ){
					DBG_ERR("Error sending packet.\n");
					return 0;
				}
				if ( __verbose ){
					if ( i == 0){
						printf("Sending packet ");
					}
					else{
						printf("Repeating packet ");
					}
					printf("from %s:%d to %s:%d\n",
						   sock_addrp,
						   ntohs(sock_addr.sin_port),						   
						   peer_addrp,
						   ntohs(peer_addr.sin_port));
					__log_buffer(buff, buff_len);
				}
			}
		}
		else{ /* port switching: free choice of source port */
			pos = rand() / ( RAND_MAX / sendings + 1);
			DBG_MSG("Position of correct port is %d\n",pos );
			port = sock_addr.sin_port; /* save correct port */
			for ( i = 0; i < sendings; i++){
				if ( i == pos ){ /* send from binded (?) port */
					n = __writen(sockfd, buff, buff_len);
					if ( n < buff_len ){
						DBG_ERR("Error sending packet.\n");
						return 0;
					}					
					if ( __verbose ){
						printf("Sending packet from %s:%d to %s:%d on correct port\n",
							   sock_addrp,
							   ntohs(port),					  
							   peer_addrp,
							   ntohs(peer_addr.sin_port));
					}
					__log_buffer(buff, buff_len);			
					sock_addr.sin_port = port;
				}
				else{
					do{
						sock_addr.sin_port = htons((int)( rand() *
														  (((float)65535-
															IPPORT_RESERVED)/
														   RAND_MAX) +
														  IPPORT_RESERVED));
					}
					while ( sock_addr.sin_port == port );
					if ( ! (tmpfd = socket(AF_INET, SOCK_DGRAM, 0)) ){
						DBG_ERR("Error getting socket for repeating packet.\n");
						return 0;
					}
					if ( bind(tmpfd,
							  (struct sockaddr*)&sock_addr,
							  sock_addr_len) ){
						DBG_ERR("Error binding temporary socket\n");
						close(tmpfd);
						return 0;
					}				
					n = sendto(tmpfd,
							   buff,
							   buff_len,
							   0,
							   (struct sockaddr*) &peer_addr,
							   peer_addr_len);
					close(tmpfd);	
					if ( n == -1 ){
						DBG_ERR("Error repeating packet via new socket.\n");
						return 0;
					}
					if ( __verbose ){
						printf("Repeating packet from %s:%d to %s:%d on wrong port\n",
							   sock_addrp,
							   ntohs(sock_addr.sin_port),					 
							   peer_addrp,
							   ntohs(peer_addr.sin_port));
						__log_buffer(buff, buff_len);						
					}
				}
			}
		}
	}  
	return 1;
}

static int
__init_con(int p_sockfd, int* cl_sockfd, int* s_sockfd, int* lose)
{
	struct sockaddr_in cl_addr, s_addr, p_addr;
	struct timeval to = {TIMEOUT_S, TIMEOUT_MS};
	socklen_t cl_addr_len, s_addr_len, p_addr_len;
	size_t buff_len = MAX_PACKET_LEN;
	uint8_t buff[buff_len];
	ssize_t n;
	int m;

	/* get packet */
	cl_addr_len = sizeof(cl_addr);
	n = recvfrom(p_sockfd,
				 buff,
				 buff_len,
				 0,
				 (struct sockaddr*) &cl_addr,
				 &cl_addr_len);
	if ( n < 0 ){
		DBG_ERR("Error reading from proxy socket.\n");
		return 0;
	}
	DBG_MSG("Red %d bytes from client socket.\n",n);
	/* now let thread go */
	if ( (m = pthread_detach(pthread_self())) ){
		DBG_ERR("Error detaching thread: %d.\n",m);
		return 0;
	}
	/* send to server */
	*s_sockfd = socket(AF_INET, SOCK_DGRAM, 0);
	if ( *s_sockfd == -1 ){
		DBG_ERR("Did not get server socket\n");
		return 0;
	}
	/* bind server side socket */
	p_addr = __p_addr;
	p_addr.sin_port = 0; // kernel may select port by itself
	p_addr_len = sizeof(p_addr);
	if ( bind(*s_sockfd, (struct sockaddr*) &p_addr, p_addr_len) ){
		DBG_ERR("Error binding socket to server port.\n");
		return 0;
	}
	s_addr = __s_addr;
	s_addr_len = sizeof(s_addr);
	if ( __select_write(*s_sockfd, NULL) <= 0){
		DBG_ERR("Error selecting server socket for write.\n");
		return 0;
	}
	n = sendto(*s_sockfd,
			   buff,
			   n,
			   0,
			   (struct sockaddr*) &s_addr,
			   s_addr_len);
	if ( n == -1 ){
		DBG_ERR("Error sending to server socket.\n");
		return 0;
	}	
	DBG_MSG("Sent %d bytes to server socket.\n",n);    
	m = __select_read(*s_sockfd, &to);
	if ( m > 0 ){
		/* server has answered */
		DBG_MSG("Server answered connection attempt.\n");
		n = recvfrom(*s_sockfd,
					 buff,
					 buff_len,
					 0,
					 (struct sockaddr*) &s_addr,
					 &s_addr_len);
		if ( n < 0 ){
			DBG_ERR("Error reading from server socket.\n");
			return 0;
		}
		DBG_MSG("Answer with %d bytes from port %d.\n",
				n, ntohs(s_addr.sin_port));
		/* connect server side socket  */
		if ( connect(*s_sockfd, (struct sockaddr*)&s_addr, s_addr_len)){
			DBG_ERR("Unable to connect server socket to server.\n");
			return 0;
		}
		/* create, bind and connect client side socket */
		*cl_sockfd = socket(AF_INET, SOCK_DGRAM, 0);
		if ( ! cl_sockfd ){
			DBG_ERR("Did not get client socket\n");
			return 0;
		}
		if ( bind(*cl_sockfd, (struct sockaddr*) &p_addr, p_addr_len) ){
			DBG_ERR("Error binding socket to client port.\n");
			return 0;
		}
		if ( getsockname(*cl_sockfd, (struct sockaddr*) &p_addr, &p_addr_len) < 0){
			DBG_ERR("Unable to get proxy address.\n");
			return 0;
		}
		DBG_MSG("Bind proxy to %s:%d\n", 
				inet_ntoa(p_addr.sin_addr), 
				ntohs(p_addr.sin_port)); 
		if ( connect(*cl_sockfd, (struct sockaddr*)&cl_addr, cl_addr_len)){
			DBG_ERR("Unable to connect client socket to client.\n");
			return 0;
		}
		DBG_MSG("Connected client socket %d to client %s:%d.\n",
				*cl_sockfd,
				inet_ntoa(cl_addr.sin_addr),
				ntohs(cl_addr.sin_port));
		if ( __verbose ){
			printf("Initiating new transfer on socket %d for client %s on port %d\n",
				   *cl_sockfd,
				   inet_ntoa(cl_addr.sin_addr),
				   ntohs(cl_addr.sin_port));
		}
		/* send it to client */
		if ( __send_error_phrone(*cl_sockfd, buff, n, lose) < 0 ){
			DBG_ERR("Error sending to client socket.\n");
			return 0;
		}
	}
	else if ( m == 0 ){ /*timeout waiting for server */
		DBG_MSG("Timeout while waiting for server to answer request.\n");
		return 0;
	}
	else{
		DBG_MSG("Error while waiting for server to answer request.\n");
		return 0;	
	}
	return 1;
}

static void*
__proxy( void* p_sockfd)
{
	int s_sockfd, cl_sockfd;
	struct timeval to_tmp = __to;
	fd_set rset;
	int maxfdp1;
	size_t buff_len = MAX_PACKET_LEN;
	uint8_t buff[buff_len];
	ssize_t n;
	int m, ret = 0;
	int lose;
        
	if( __init_con( (int)p_sockfd, &cl_sockfd, &s_sockfd, &lose) ){
		DBG_MSG("Prepared to exchange data between client and server.\n");
		FD_ZERO(&rset);
		while (1){
			FD_SET(cl_sockfd, &rset);
			FD_SET(s_sockfd, &rset);
			maxfdp1 = max(cl_sockfd, s_sockfd)+1;
			m = select(maxfdp1, &rset, NULL, NULL, &to_tmp);
			if ( m > 0 ){
				to_tmp = __to; /* set old timeout */
				while ( m > 0 ){
					if ( FD_ISSET(cl_sockfd, &rset)){
						n = __readn(cl_sockfd, &buff, buff_len);
						if ( n < 0 ){
							DBG_ERR("Error while reading from socket.\n");
							break;
						}						
						n -= __writen(s_sockfd, buff, n);
						if ( n != 0  ){
							DBG_ERR("Error sending packet.\n");
							return 0;
						}
						DBG_MSG("Transfering packet from client to server.\n");
					}
					else if ( FD_ISSET(s_sockfd, &rset)){
						n = __readn(s_sockfd, &buff, buff_len);
						if ( n < 0 ){
							DBG_ERR("Error while reading from socket.\n");
							break;
						}   
						/* simulate line errors to client */
						if ( __send_error_phrone(cl_sockfd,
												 buff,
												 n, &lose) < 0){
							DBG_ERR("Error sending to client socket.\n");
							return 0;	    
						}
					}
					else{ /* huh? */
						DBG_ERR("Select ready but none of my descriptors included.\n");
						break;
					}
					m--;
				}
			}
			else if ( m == 0 ){ /* timeout */
				DBG_MSG("Timeout selecting from sockets.\n");
				ret = 1; /* this is not a failure, indeed .. */
				break;
			}
			else{
				DBG_ERR("Error while selecting from sockets.\n");
				break; /* .. but this */			
			}
		}
	}
	else{
		DBG_ERR("Error initializing client-server-connection.\n");
	}
	/* clean up  */
	printf("Finishing connection on socket %d\n", cl_sockfd);	
	close(cl_sockfd);
	close(s_sockfd);
	return (NULL);
}

static int
__wait_and_listen()
{
	int p_sockfd;
	int n, t;
	pthread_t t_id;

	p_sockfd = socket(AF_INET, SOCK_DGRAM, 0);
	if ( ! p_sockfd){
		DBG_LMSG("Did not get proxy socket\n");
		return 0;
	}
	DBG_MSG("Proxy socket is %d .\n",p_sockfd);
	if ( bind(p_sockfd,
			  (const struct sockaddr*) &__p_addr,
			  sizeof(__p_addr)) ){
		DBG_ERR("Error binding socket %d to proxy address.\n", p_sockfd);
		return 0;
	}    
	while ( 1 ){
		n = __select_read(p_sockfd, NULL);
		if ( n >= 0 ){
			DBG_MSG("Incoming packet on proxy port signalized.\n");
			if ( ( t = pthread_create(&t_id,
									  NULL,
									  __proxy,
									  (void*) p_sockfd) ) ) {
				DBG_ERR("Error creating proxy thread: %d\n",t);
				return 0;
			}
			DBG_MSG("Created new thread to handle request.\n");
		}
		else{ /* timeout or error, both means an error */
			DBG_ERR("Error selecting from proxy socket.\n");
			return 0;
		}
	}
	return 1;
}

static int
__init(struct properties* prop)
{
	if ( prop){
		/* initialize global constants */
		__p_addr.sin_family = AF_INET;
		if ( prop->p_addr ){
			if ( ! inet_pton(AF_INET,prop->p_addr,&(__p_addr.sin_addr))){
				printf("Invalid proxy inet address %s, conversion failed.\n",
					   prop->p_addr );
				return 0;
			}
			DBG_MSG("Listening on ip %s.\n", prop->p_addr);		
		}
		else{
			__p_addr.sin_addr.s_addr = htonl(INADDR_ANY);
			DBG_MSG("Listening on all interfaces.\n");
		}
		__p_addr.sin_port = htons(prop->p_port);
		DBG_MSG("Listening on port %d.\n", prop->p_port);
		__s_addr.sin_family = AF_INET;
		if ( ! inet_pton(AF_INET,prop->s_addr,&(__s_addr.sin_addr))){
			printf("Invalid server inet address %s, conversion failed.\n",
				   prop->s_addr );
			return 0;
		}
		DBG_MSG("Transfering all data to server %s.\n",prop->s_addr);
		__s_addr.sin_port = htons(prop->s_port);
		DBG_MSG("Will communicate with server on port %d.\n", prop->s_port);
		__to.tv_sec = prop->to_s;
		__to.tv_usec = prop->to_us;
		return 1;
	}
	else{
		DBG_ERR("Missing properties for initializing.\n");
		return 0;
	}
}

static int
__get_defaults(struct properties* prop)
{
	if ( prop){
		prop->s_addr = SERV_ADDR;
		prop->s_port = SERV_PORT;
		prop->p_addr = PROXY_ADDR;
		prop->p_port = PROXY_PORT;
		prop->to_s = TIMEOUT_S;
		prop->to_us = TIMEOUT_MS;
		return 1;
	}
	return 0;
}

static int
__usage(char* arg0)
{
    printf("Usage: %s [OPTION] \n", arg0);
    printf("\t -a ip \t\t\t bind address. Default any\n");
    printf("\t -p port \t\t listening port. Default %d\n", (int)PROXY_PORT);
    printf("\t -s ip \t\t\t server address. Default %s\n", SERV_ADDR);
    printf("\t -o port \t\t port of server. Default %d\n", (int)SERV_PORT);
    printf("\t -t second \t\t timeout in sec. Default %d:%ds\n", 
		   TIMEOUT_S, 
		   TIMEOUT_MS );;
    printf("\t -v \t\t\t be more verbose\n");
    printf("\t -l percent \t\t propability of loosing a packet. Default %d\n",
		   LOST_PRB);
    printf("\t -r percent \t\t propability of repeating a packet. Default %d\n",	   
		   REPEAT_PRB);
    printf("\t -b len \t\t max burst len of loosing packets. Default %d\n",
		   LOST_BURST_LEN);
    printf("\t -d len \t\t max burst len of repeating packets. Default %d\n",
		   REPEAT_BURST_LEN);    
    printf("\t -w \t\t\t Enable sending packets to wrong ports\n");
	printf("\t -h \t\t\t show this help\n");
    return 1;
}

int
main(int argc, char** argv)
{
	struct properties prop;
	int c;
	extern char *optarg;
	extern int optind, optopt, opterr;
	char* cmd_args = ":a:p:s:o:t:vhl:r:b:d:w";
	int ret = 1;
   
	if ( ! __get_defaults(&prop)){
		printf("Error loading default values.\n");
		return 0;
	}
	while ((c = getopt(argc, argv, cmd_args)) != -1) {
		switch(c) {
		case 'a':
			prop.p_addr = optarg;
			DBG_MSG("Set listening address to %s\n",prop.s_addr);
			break;
		case 'p':
			prop.p_port = htons(atoi(optarg));
			DBG_MSG("Set listening port to %d\n",atoi(optarg));
			break;
		case 's':
			prop.s_addr = optarg;
			DBG_MSG("Set server address to %s\n",prop.s_addr);
			break;
		case 'o':
			prop.s_port = htons(atoi(optarg));
			DBG_MSG("Set server port to %d\n",atoi(optarg));
			break;
		case 't':
			prop.to_s = atoi(optarg);
			DBG_MSG("Set timeout to %ld:%ld\n", prop.to_s, prop.to_us);
			break;
		case 'v':
			__verbose++;
			DBG_MSG("Raised verboise level by one to %d\n",__verbose);
			break;
		case 'h':
			ret = 0;
			break;			
		case 'l':
			__lose_prb = atoi(optarg);
			if ( __lose_prb > 100 || __lose_prb < 0 ){
				printf("Option -%c accepts values between 0 and 100.\n", optopt);
				ret = 0;
				break;
			}
			DBG_MSG("Set lose propability to %d percent\n",__lose_prb);
			break;
		case 'r':
			__repeat_prb = atoi(optarg);
			if ( __repeat_prb > 100 ||  __repeat_prb < 0 ){
				printf("Option -%c accepts values between 0 and 100.\n", optopt);
				ret = 0;
				break;
			}
			DBG_MSG("Set repeat propability to %d percent\n",__repeat_prb);
			break;
		case 'b':
			__lose_burst_len = atoi(optarg);
			if ( __lose_burst_len < 0 ){
				printf("Option -%c does not accept negative arguments.\n", optopt);
				ret = 0;
				break;
			}
			DBG_MSG("Set lose burst len to %d\n",__lose_burst_len);
			break;
		case 'd':
			__repeat_burst_len = atoi(optarg);
			if ( __repeat_burst_len < 0 ){
				printf("Option -%c does not accept negative arguments.\n", optopt);
				ret = 0;
				break;
			}
			DBG_MSG("Set repeat burst len to %d\n",__repeat_burst_len);
			break;
		case 'w':
			__repeat_port_switch++;
			DBG_MSG("Enabled port switching on repeated packets.\n");
			break;
		case ':':
			printf("Missing argument for -%c \n", optopt);
			ret = 0;
			break;
		case '?':
			printf("Unknown arg %c\n", optopt);
			ret = 0;
			break;
		}
	}
	if ( ! ret ){
		__usage(argv[0]);
		return ret;
	}
	if ( ! __init(&prop)){
		DBG_MSG("Error initializing proxy.\n");
		__usage(argv[0]);
		return 0;
	}
	if ( ! __wait_and_listen()){
		DBG_MSG("Error wait and listening.\n");
		return 0;
	}
	return 1;
}








